@@ -1,4 +1,5 @@
 /******************************************************************************
+ * $Id: gdalsievefilter.cpp 21213 2010-12-08 16:43:11Z warmerdam $
  *
  * Project:  GDAL
  * Purpose:  Raster to Polygon Converter
@@ -30,9 +31,10 @@
 #include "cpl_conv.h"
 #include <vector>
 
-CPL_CVSID("$Id: gdalsievefilter.cpp 15701 2008-11-10 15:40:02Z warmerdam $");
+CPL_CVSID("$Id: gdalsievefilter.cpp 21213 2010-12-08 16:43:11Z warmerdam $");
 
 #define GP_NODATA_MARKER -51502112
+#define MY_MAX_INT 2147483647
 
 /*
  * General Plan
@@ -62,14 +64,12 @@ CPL_CVSID("$Id: gdalsievefilter.cpp 15701 2008-11-10 15:40:02Z warmerdam $");
 /************************************************************************/
 
 static CPLErr 
-GPMaskImageData( GDALRasterBandH hMaskBand, int iY, int nXSize, 
+GPMaskImageData( GDALRasterBandH hMaskBand, GByte *pabyMaskLine, int iY, int nXSize, 
                  GInt32 *panImageLine )
 
 {
-    GByte *pabyMaskLine;
     CPLErr eErr;
 
-    pabyMaskLine = (GByte *) CPLMalloc(nXSize);
     eErr = GDALRasterIO( hMaskBand, GF_Read, 0, iY, nXSize, 1, 
                          pabyMaskLine, nXSize, 1, GDT_Byte, 0, 0 );
     if( eErr == CE_None )
@@ -82,8 +82,6 @@ GPMaskImageData( GDALRasterBandH hMaskBand, int iY, int nXSize,
         }
     }
 
-    CPLFree( pabyMaskLine );
-
     return eErr;
 }
 
@@ -189,6 +187,9 @@ GDALSieveFilter( GDALRasterBandH hSrcBand, GDALRasterBandH hMaskBand,
                  void * pProgressArg )
 
 {
+    VALIDATE_POINTER1( hSrcBand, "GDALSieveFilter", CE_Failure );
+    VALIDATE_POINTER1( hDstBand, "GDALSieveFilter", CE_Failure );
+
     if( pfnProgress == NULL )
         pfnProgress = GDALDummyProgress;
 
@@ -198,11 +199,27 @@ GDALSieveFilter( GDALRasterBandH hSrcBand, GDALRasterBandH hMaskBand,
     CPLErr eErr = CE_None;
     int nXSize = GDALGetRasterBandXSize( hSrcBand );
     int nYSize = GDALGetRasterBandYSize( hSrcBand );
-    GInt32 *panLastLineVal = (GInt32 *) CPLMalloc(4 * nXSize);
-    GInt32 *panThisLineVal = (GInt32 *) CPLMalloc(4 * nXSize);
-    GInt32 *panLastLineId =  (GInt32 *) CPLMalloc(4 * nXSize);
-    GInt32 *panThisLineId =  (GInt32 *) CPLMalloc(4 * nXSize);
-    GInt32 *panThisLineWriteVal = (GInt32 *) CPLMalloc(4 * nXSize);
+    GInt32 *panLastLineVal = (GInt32 *) VSIMalloc2(sizeof(GInt32), nXSize);
+    GInt32 *panThisLineVal = (GInt32 *) VSIMalloc2(sizeof(GInt32), nXSize);
+    GInt32 *panLastLineId =  (GInt32 *) VSIMalloc2(sizeof(GInt32), nXSize);
+    GInt32 *panThisLineId =  (GInt32 *) VSIMalloc2(sizeof(GInt32), nXSize);
+    GInt32 *panThisLineWriteVal = (GInt32 *) VSIMalloc2(sizeof(GInt32), nXSize);
+    GByte *pabyMaskLine = (hMaskBand != NULL) ? (GByte *) VSIMalloc(nXSize) : NULL;
+    if (panLastLineVal == NULL || panThisLineVal == NULL ||
+        panLastLineId == NULL || panThisLineId == NULL ||
+        panThisLineWriteVal == NULL ||
+        (hMaskBand != NULL && pabyMaskLine == NULL))
+    {
+        CPLError(CE_Failure, CPLE_OutOfMemory,
+                 "Could not allocate enough memory for temporary buffers");
+        CPLFree( panThisLineId );
+        CPLFree( panLastLineId );
+        CPLFree( panThisLineVal );
+        CPLFree( panLastLineVal );
+        CPLFree( panThisLineWriteVal );
+        CPLFree( pabyMaskLine );
+        return CE_Failure;
+    }
 
 /* -------------------------------------------------------------------- */
 /*      The first pass over the raster is only used to build up the     */
@@ -221,7 +238,7 @@ GDALSieveFilter( GDALRasterBandH hSrcBand, GDALRasterBandH hMaskBand,
             panThisLineVal, nXSize, 1, GDT_Int32, 0, 0 );
         
         if( eErr == CE_None && hMaskBand != NULL )
-            eErr = GPMaskImageData( hMaskBand, iY, nXSize, panThisLineVal );
+            eErr = GPMaskImageData( hMaskBand, pabyMaskLine, iY, nXSize, panThisLineVal );
 
         if( iY == 0 )
             oFirstEnum.ProcessLine( 
@@ -243,7 +260,8 @@ GDALSieveFilter( GDALRasterBandH hSrcBand, GDALRasterBandH hMaskBand,
             iPoly = panThisLineId[iX]; 
 
             CPLAssert( iPoly >= 0 );
-            anPolySizes[iPoly] += 1;
+            if( anPolySizes[iPoly] < MY_MAX_INT )
+                anPolySizes[iPoly] += 1;
         }
 
 /* -------------------------------------------------------------------- */
@@ -284,7 +302,14 @@ GDALSieveFilter( GDALRasterBandH hSrcBand, GDALRasterBandH hMaskBand,
     {
         if( oFirstEnum.panPolyIdMap[iPoly] != iPoly )
         {
-            anPolySizes[oFirstEnum.panPolyIdMap[iPoly]] += anPolySizes[iPoly];
+            GIntBig nSize = anPolySizes[oFirstEnum.panPolyIdMap[iPoly]];
+
+            nSize += anPolySizes[iPoly];
+            
+            if( nSize > MY_MAX_INT )
+                nSize = MY_MAX_INT;
+
+            anPolySizes[oFirstEnum.panPolyIdMap[iPoly]] = (int)nSize;
             anPolySizes[iPoly] = 0;
         }
     }
@@ -314,7 +339,7 @@ GDALSieveFilter( GDALRasterBandH hSrcBand, GDALRasterBandH hMaskBand,
                              panThisLineVal, nXSize, 1, GDT_Int32, 0, 0 );
 
         if( eErr == CE_None && hMaskBand != NULL )
-            eErr = GPMaskImageData( hMaskBand, iY, nXSize, panThisLineVal );
+            eErr = GPMaskImageData( hMaskBand, pabyMaskLine, iY, nXSize, panThisLineVal );
 
         if( eErr != CE_None )
             continue;
@@ -475,7 +500,7 @@ GDALSieveFilter( GDALRasterBandH hSrcBand, GDALRasterBandH hMaskBand,
         memcpy( panThisLineWriteVal, panThisLineVal, 4 * nXSize );
 
         if( eErr == CE_None && hMaskBand != NULL )
-            eErr = GPMaskImageData( hMaskBand, iY, nXSize, panThisLineVal );
+            eErr = GPMaskImageData( hMaskBand, pabyMaskLine, iY, nXSize, panThisLineVal );
 
         if( eErr != CE_None )
             continue;
@@ -546,6 +571,8 @@ GDALSieveFilter( GDALRasterBandH hSrcBand, GDALRasterBandH hMaskBand,
     CPLFree( panLastLineId );
     CPLFree( panThisLineVal );
     CPLFree( panLastLineVal );
+    CPLFree( panThisLineWriteVal );
+    CPLFree( pabyMaskLine );
 
     return eErr;
 }
