@@ -1,5 +1,5 @@
 /* ****************************************************************************
- * $Id: dumpoverviews.cpp 11991 2007-08-29 12:56:59Z warmerdam $
+ * $Id: dumpoverviews.cpp 20991 2010-10-27 20:13:47Z rouault $
  *
  * Project:  GDAL Utilities
  * Purpose:  Dump overviews to external files.
@@ -32,7 +32,7 @@
 #include "gdal_priv.h"
 #include "ogr_spatialref.h"
 
-CPL_CVSID("$Id: dumpoverviews.cpp 11991 2007-08-29 12:56:59Z warmerdam $");
+CPL_CVSID("$Id: dumpoverviews.cpp 20991 2010-10-27 20:13:47Z rouault $");
 
 static void DumpBand( GDALDatasetH hBaseDS, GDALRasterBandH hBand,
                       const char *pszName );
@@ -113,6 +113,12 @@ int main( int argc, char ** argv )
         for( iOverview = 0; iOverview < nOverviewCount; iOverview++ )
         {
             GDALRasterBandH hSrcOver = GDALGetOverview( hBaseBand, iOverview );
+            
+            if (hSrcOver == NULL)
+            {
+                fprintf(stderr, "skipping overview %d as being null\n", iOverview);
+                continue;
+            }
 
 /* -------------------------------------------------------------------- */
 /*      Is this a requested overview?                                   */
@@ -136,9 +142,18 @@ int main( int argc, char ** argv )
 /* -------------------------------------------------------------------- */
             CPLString osFilename;
             osFilename.Printf( "%s_%d_%d.tif",
-                               CPLGetBasename(pszSrcFilename), 
+                               CPLGetBasename(pszSrcFilename),
                                iBand+1, iOverview );
             DumpBand( hSrcDS, hSrcOver, osFilename );
+
+            if( bMasks )
+            {
+                CPLString osFilename;
+                osFilename.Printf( "%s_%d_%d_mask.tif",
+                                CPLGetBasename(pszSrcFilename),
+                                iBand+1, iOverview );
+                DumpBand( hSrcDS, GDALGetMaskBand(hSrcOver), osFilename );
+            }
         }
 
 /* -------------------------------------------------------------------- */
@@ -148,7 +163,7 @@ int main( int argc, char ** argv )
         {
             CPLString osFilename;
             osFilename.Printf( "%s_%d_mask.tif",
-                               CPLGetBasename(pszSrcFilename), 
+                               CPLGetBasename(pszSrcFilename),
                                iBand+1 );
             DumpBand( hSrcDS, GDALGetMaskBand(hBaseBand), osFilename );
         }
@@ -158,6 +173,8 @@ int main( int argc, char ** argv )
 
     CSLDestroy( argv );
     GDALDestroyDriverManager();
+    
+    return 0;
 }
 
 /************************************************************************/
