@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrinfo.cpp 14818 2008-07-05 10:15:08Z rouault $
+ * $Id: ogrinfo.cpp 20589 2010-09-12 16:47:56Z rouault $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Simple client for viewing OGR driver data.
@@ -27,13 +27,14 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
+#include "ogr_api.h"
 #include "ogrsf_frmts.h"
 #include "ogr_p.h"
 #include "cpl_conv.h"
 #include "cpl_string.h"
 #include "cpl_multiproc.h"
 
-CPL_CVSID("$Id: ogrinfo.cpp 14818 2008-07-05 10:15:08Z rouault $");
+CPL_CVSID("$Id: ogrinfo.cpp 20589 2010-09-12 16:47:56Z rouault $");
 
 int     bReadOnly = FALSE;
 int     bVerbose = TRUE;
@@ -59,6 +60,7 @@ int main( int nArgc, char ** papszArgv )
     int         nRepeatCount = 1, bAllLayers = FALSE;
     const char  *pszSQLStatement = NULL;
     const char  *pszDialect = NULL;
+    int          nRet = 0;
     
     /* Check strict compilation and runtime library version as we use C++ API */
     if (! GDAL_CHECK_VERSION(papszArgv[0]))
@@ -86,7 +88,7 @@ int main( int nArgc, char ** papszArgv )
         }
         else if( EQUAL(papszArgv[iArg],"-ro") )
             bReadOnly = TRUE;
-        else if( EQUAL(papszArgv[iArg],"-q") )
+        else if( EQUAL(papszArgv[iArg],"-q") || EQUAL(papszArgv[iArg],"-quiet"))
             bVerbose = FALSE;
         else if( EQUAL(papszArgv[iArg],"-fid") && iArg < nArgc-1 )
             nFetchFID = atoi(papszArgv[++iArg]);
@@ -197,7 +199,8 @@ int main( int nArgc, char ** papszArgv )
             printf( "  -> %s\n", poR->GetDriver(iDriver)->GetName() );
         }
 
-        exit( 1 );
+        nRet = 1;
+        goto end;
     }
 
     CPLAssert( poDriver != NULL);
@@ -242,41 +245,65 @@ int main( int nArgc, char ** papszArgv )
         }
     }
 
-/* -------------------------------------------------------------------- */
-/*      Process each data source layer.                                 */
-/* -------------------------------------------------------------------- */
     CPLDebug( "OGR", "GetLayerCount() = %d\n", poDS->GetLayerCount() );
 
     for( int iRepeat = 0; iRepeat < nRepeatCount; iRepeat++ )
     {
-        for( int iLayer = 0; iLayer < poDS->GetLayerCount(); iLayer++ )
+        if ( CSLCount(papszLayers) == 0 )
         {
-            OGRLayer        *poLayer = poDS->GetLayer(iLayer);
-
-            if( poLayer == NULL )
+/* -------------------------------------------------------------------- */ 
+/*      Process each data source layer.                                 */ 
+/* -------------------------------------------------------------------- */ 
+            for( int iLayer = 0; iLayer < poDS->GetLayerCount(); iLayer++ )
             {
-                printf( "FAILURE: Couldn't fetch advertised layer %d!\n",
-                        iLayer );
-                exit( 1 );
+                OGRLayer        *poLayer = poDS->GetLayer(iLayer);
+
+                if( poLayer == NULL )
+                {
+                    printf( "FAILURE: Couldn't fetch advertised layer %d!\n",
+                            iLayer );
+                    exit( 1 );
+                }
+
+                if (!bAllLayers)
+                {
+                    printf( "%d: %s",
+                            iLayer+1,
+                            poLayer->GetName() );
+
+                    if( poLayer->GetGeomType() != wkbUnknown )
+                        printf( " (%s)", 
+                                OGRGeometryTypeToName( 
+                                    poLayer->GetGeomType() ) );
+
+                    printf( "\n" );
+                }
+                else
+                {
+                    if( iRepeat != 0 )
+                        poLayer->ResetReading();
+
+                    ReportOnLayer( poLayer, pszWHERE, poSpatialFilter );
+                }
             }
-
-            if( CSLCount(papszLayers) == 0 && !bAllLayers )
+        }
+        else
+        {
+/* -------------------------------------------------------------------- */ 
+/*      Process specified data source layers.                           */ 
+/* -------------------------------------------------------------------- */ 
+            char** papszIter = papszLayers;
+            for( ; *papszIter != NULL; papszIter++ )
             {
-                printf( "%d: %s",
-                        iLayer+1,
-                        poLayer->GetLayerDefn()->GetName() );
+                OGRLayer        *poLayer = poDS->GetLayerByName(*papszIter);
 
-                if( poLayer->GetLayerDefn()->GetGeomType() != wkbUnknown )
-                    printf( " (%s)", 
-                            OGRGeometryTypeToName( 
-                                poLayer->GetLayerDefn()->GetGeomType() ) );
+                if( poLayer == NULL )
+                {
+                    printf( "FAILURE: Couldn't fetch requested layer %s!\n",
+                            *papszIter );
+                    exit( 1 );
+                }
 
-                printf( "\n" );
-            }
-            else if( bAllLayers 
-                     || CSLFindString( papszLayers,
-                                   poLayer->GetLayerDefn()->GetName() ) != -1 )
-            {
                 if( iRepeat != 0 )
                     poLayer->ResetReading();
 
@@ -288,21 +315,17 @@ int main( int nArgc, char ** papszArgv )
 /* -------------------------------------------------------------------- */
 /*      Close down.                                                     */
 /* -------------------------------------------------------------------- */
+end:
     CSLDestroy( papszArgv );
     CSLDestroy( papszLayers );
     CSLDestroy( papszOptions );
-    delete poDS;
+    OGRDataSource::DestroyDataSource( poDS );
     if (poSpatialFilter)
-        delete poSpatialFilter;
+        OGRGeometryFactory::destroyGeometry( poSpatialFilter );
 
-    delete OGRSFDriverRegistrar::GetRegistrar();
-    OSRCleanup();
-    CPLFinderClean();
-    VSICleanupFileManager();
-    CPLFreeConfig();
-    CPLCleanupTLS();
+    OGRCleanupAll();
 
-    return 0;
+    return nRet;
 }
 
 /************************************************************************/
@@ -314,7 +337,7 @@ static void Usage()
 {
     printf( "Usage: ogrinfo [--help-general] [-ro] [-q] [-where restricted_where]\n"
             "               [-spat xmin ymin xmax ymax] [-fid fid]\n"
-            "               [-sql statement] [-al] [-so] [-fields={YES/NO}]\n"
+            "               [-sql statement] [-dialect sql_dialect] [-al] [-so] [-fields={YES/NO}]\n"
             "               [-geom={YES/NO/SUMMARY}][--formats]\n"
             "               datasource_name [layer [layer ...]]\n");
     exit( 1 );
@@ -344,12 +367,12 @@ static void ReportOnLayer( OGRLayer * poLayer, const char *pszWHERE,
 /* -------------------------------------------------------------------- */
     printf( "\n" );
     
-    printf( "Layer name: %s\n", poDefn->GetName() );
+    printf( "Layer name: %s\n", poLayer->GetName() );
 
     if( bVerbose )
     {
         printf( "Geometry: %s\n", 
-                OGRGeometryTypeToName( poDefn->GetGeomType() ) );
+                OGRGeometryTypeToName( poLayer->GetGeomType() ) );
         
         printf( "Feature Count: %d\n", poLayer->GetFeatureCount() );
         
@@ -402,7 +425,7 @@ static void ReportOnLayer( OGRLayer * poLayer, const char *pszWHERE,
         while( (poFeature = poLayer->GetNextFeature()) != NULL )
         {
             poFeature->DumpReadable( NULL, papszOptions );
-            delete poFeature;
+            OGRFeature::DestroyFeature( poFeature );
         }
     }
     else if( nFetchFID != OGRNullFID )
@@ -416,7 +439,7 @@ static void ReportOnLayer( OGRLayer * poLayer, const char *pszWHERE,
         else
         {
             poFeature->DumpReadable( NULL, papszOptions );
-            delete poFeature;
+            OGRFeature::DestroyFeature( poFeature );
         }
     }
 }
