@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gsagdataset.cpp 16925 2009-05-03 14:16:49Z rouault $
+ * $Id: gsagdataset.cpp 20996 2010-10-28 18:38:15Z rouault $
  *
  * Project:  GDAL
  * Purpose:  Implements the Golden Software ASCII Grid Format.
@@ -49,7 +49,7 @@
 # define INT_MAX 2147483647
 #endif /* INT_MAX */
 
-CPL_CVSID("$Id: gsagdataset.cpp 16925 2009-05-03 14:16:49Z rouault $");
+CPL_CVSID("$Id: gsagdataset.cpp 20996 2010-10-28 18:38:15Z rouault $");
 
 CPL_C_START
 void	GDALRegister_GSAG(void);
@@ -71,9 +71,9 @@ class GSAGDataset : public GDALPamDataset
     static const int nFIELD_PRECISION;
     static const size_t nMAX_HEADER_SIZE;
 
-    static CPLErr ShiftFileContents( FILE *, vsi_l_offset, int, const char * );
+    static CPLErr ShiftFileContents( VSILFILE *, vsi_l_offset, int, const char * );
 
-    FILE	*fp;
+    VSILFILE	*fp;
     size_t	 nMinMaxZOffset;
     char	 szEOL[3];
 
@@ -1163,7 +1163,7 @@ CPLErr GSAGDataset::SetGeoTransform( double *padfGeoTransform )
 /************************************************************************/
 /*                         ShiftFileContents()                          */
 /************************************************************************/
-CPLErr GSAGDataset::ShiftFileContents( FILE *fp, vsi_l_offset nShiftStart,
+CPLErr GSAGDataset::ShiftFileContents( VSILFILE *fp, vsi_l_offset nShiftStart,
 				       int nShiftSize, const char *pszEOL )
 {
     /* nothing to do for zero-shift */
@@ -1515,7 +1515,7 @@ GDALDataset *GSAGDataset::CreateCopy( const char *pszFilename,
         return NULL;
     }
 
-    FILE *fp = VSIFOpenL( pszFilename, "w+b" );
+    VSILFILE *fp = VSIFOpenL( pszFilename, "w+b" );
 
     if( fp == NULL )
     {
