@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: geo_normalize.c 1493 2008-11-28 02:48:56Z warmerdam $
+ * $Id: geo_normalize.c 1685 2009-11-11 17:08:09Z warmerdam $
  *
  * Project:  libgeotiff
  * Purpose:  Code to normalize PCS and other composite codes in a GeoTIFF file.
@@ -25,193 +25,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.53  2008/07/03 18:36:31  fwarmerdam
- * Fix potential buffer overflow in GTIFAngleStringToDD.
- * http://trac.osgeo.org/gdal/ticket/2228
- *
- * Revision 1.52  2008/01/31 19:47:57  fwarmerdam
- * Ignore GCS values less than 1 as a sanity measure
- *
- * Revision 1.51  2007/12/11 17:58:34  fwarmerdam
- * Add EPSG 9822 (Albers Equal Area) support from EPSG
- *
- * Revision 1.50  2007/07/28 13:55:21  fwarmerdam
- * Fix name for GCS_WGS_72 per gdal bug #1715.
- *
- * Revision 1.49  2007/07/20 18:10:41  fwarmerdam
- * Pre-search pcs.override.csv and gcs.override.csv.
- *
- * Revision 1.48  2007/06/06 02:17:04  fwarmerdam
- * added builtin known values for foot and us survey foot
- *
- * Revision 1.47  2007/03/13 18:04:33  fwarmerdam
- * added new zealand map grid support per bug 1519
- *
- * Revision 1.46  2006/04/11 19:25:06  fwarmerdam
- * Be careful about falling back to gdal_datum.csv as it can interfere
- * with incode datum.csv support.
- *
- * Revision 1.45  2005/03/15 16:01:18  fwarmerdam
- * zero inv flattening interpreted as sphere
- *
- * Revision 1.44  2005/03/04 04:32:37  fwarmerdam
- * added cylindricalequalarea support
- *
- * Revision 1.43  2005/03/04 04:02:40  fwarmerdam
- * Fixed initialization of dfStdParallel2 for AEA and EC.
- *
- * Revision 1.42  2005/02/17 01:21:38  fwarmerdam
- * fixed handling of ProjFalseOrigin{Easting,Northing}GeoKey
- *
- * Revision 1.41  2004/12/01 22:06:42  fwarmerdam
- * bug 698: GTIFGetGCSInfo should not fail on missing pm if pm info not req.
- *
- * Revision 1.40  2004/07/09 17:27:37  warmerda
- * Added 9122 as an alias for simple degrees.
- *
- * Revision 1.39  2004/06/07 12:57:13  warmerda
- * fallback to using gdal_datum.csv if datum.csv not found
- *
- * Revision 1.38  2004/03/19 12:20:40  dron
- * Initialize projection parameters in GTIFFetchProjParms() before using.
- *
- * Revision 1.37  2003/07/08 17:31:30  warmerda
- * cleanup various warnings
- *
- * Revision 1.36  2003/01/28 18:31:58  warmerda
- * Default dfInDegrees in GTIFAngleToDD().
- *
- * Revision 1.35  2003/01/15 04:39:16  warmerda
- * Added GTIFDeaccessCSV
- *
- * Revision 1.34  2003/01/15 03:37:40  warmerda
- * added GTIFFreeMemory()
- *
- * Revision 1.33  2002/12/05 19:21:01  warmerda
- * fixed dfInDegrees to actually be in degrees, not radians!
- *
- * Revision 1.32  2002/11/30 16:01:11  warmerda
- * fixed some problems in GTIFGetUOMAngleInfo
- *
- * Revision 1.31  2002/11/30 15:44:35  warmerda
- * fixed GetCTParms EPSG code mappings
- *
- * Revision 1.30  2002/11/28 22:27:42  warmerda
- * preliminary upgrade to EPSG 6.2.2 tables
- *
- * Revision 1.29  2002/06/19 03:51:15  warmerda
- * migrated cpl_csv.h into cpl_serv.h
- *
- * Revision 1.28  2002/01/03 21:28:25  warmerda
- * call CSVDeaccess(NULL) at end of GTIFPrintDefn()
- *
- * Revision 1.27  2001/04/17 13:41:10  warmerda
- * fix memory leaks in GTIFPrintDefn()
- *
- * Revision 1.26  2001/04/17 13:23:07  warmerda
- * added support for reading custom ellipsoid definitions
- *
- * Revision 1.25  2001/03/05 04:55:26  warmerda
- * CVSDeaccess at end of GTIFGetDefn to avoid file leak
- *
- * Revision 1.24  2001/03/05 03:26:29  warmerda
- * fixed memory leaks in GTIFPrintDefn()
- *
- * Revision 1.23  2001/02/23 13:49:48  warmerda
- * Fixed GTIFPrintDefn() to use fprintf( fp ), instead of printf().
- *
- * Revision 1.22  2000/10/13 14:30:57  warmerda
- * fixed LCC parm order when parameters read directly from geotiff file
- *
- * Revision 1.21  2000/09/15 19:30:14  warmerda
- * report units of linear proj parms
- *
- * Revision 1.20  2000/09/15 18:21:07  warmerda
- * Fixed order of parameters for LCC 2SP.  When parameters
- * were read from EPSG CSV files the standard parallels and origin
- * were mixed up.  This affects alot of state plane zones!
- *
- * Revision 1.19  2000/06/09 14:05:43  warmerda
- * added default knowledge of NAD27/NAD83/WGS72/WGS84
- *
- * Revision 1.18  1999/12/10 21:28:12  warmerda
- * fixed Stereographic to look for ProjCenterLat/Long
- *
- * Revision 1.17  1999/12/10 20:06:58  warmerda
- * fixed up scale geokey used for a couple of projections
- *
- * Revision 1.16  1999/12/10 19:50:21  warmerda
- * Added EquidistantConic support, fixed return of StdParallel2GeoKey for
- * LCC2, and Albers.
- *
- * Revision 1.15  1999/12/10 19:39:26  warmerda
- * Fixed bug setting the false northing for files with
- * ProjCenterNorthingGeoKey set in GTIFGetDefn().
- *
- * Revision 1.14  1999/09/17 14:58:37  warmerda
- * Added ProjRectifiedGridAngleGeoKey(3096) and support for it's
- * use with Oblique Mercator in geo_normalize.c.
- *
- * Revision 1.13  1999/09/17 00:55:26  warmerda
- * added GTIFGetUOMAngleInfo(), and UOMAngle in GTIFDefn
- *
- * Revision 1.12  1999/09/15 18:51:31  warmerda
- * Map 9808 to TM South Oriented, not TM Modified Alaska.
- *
- * Revision 1.11  1999/09/15 16:44:06  warmerda
- * Change meter to metre to match EPSG database in GTIFGetUOMLengthInfo()
- * shortcut.
- *
- * Revision 1.10  1999/09/15 16:35:15  warmerda
- * Fixed the fractions of second handling properly in GTIFAngleStringToDD().
- *
- * Revision 1.9  1999/09/15 14:24:17  warmerda
- * Fixed serious bug in geo_normalize.c with translation of
- * DD.MMSSsss values.  Return value was seriously off if any
- * fraction of a second was included in the string.
- *
- * Revision 1.8  1999/07/13 03:12:52  warmerda
- * Make scale a parameter of CT_Stereographic.
- *
- * Revision 1.7  1999/05/04 03:13:22  warmerda
- * fixed a serious bug in parsing DMSmmss.sss values, and a bug in forming DMS strings
- *
- * Revision 1.6  1999/05/03 17:50:31  warmerda
- * avoid warnings on IRIX
- *
- * Revision 1.5  1999/04/28 20:04:51  warmerda
- * Added doxygen style documentation.
- * Use GTIFPCSToMapSys() and related functions to partially normalize
- * projections when we don't have the CSV files.
- *
- * Revision 1.4  1999/03/18 21:34:59  geotiff
- * added GTIFDecToDMS
- *
- * Revision 1.3  1999/03/17 19:53:15  geotiff
- * sys includes moved to cpl_serv.h
- *
- * Revision 1.2  1999/03/10 18:24:06  geotiff
- * corrected to use int'
- *
- * Revision 1.1  1999/03/09 15:57:04  geotiff
- * New
- *
- * Revision 1.4  1999/03/03 02:29:38  warmerda
- * Define PI if not already defined.
- *
- * Revision 1.3  1999/03/02 21:10:57  warmerda
- * added lots of projections
- *
- * Revision 1.2  1999/02/24 16:24:15  warmerda
- * Continuing to evolve
- *
- * Revision 1.1  1999/02/22 18:51:08  warmerda
- * New
- *
- */
+ *****************************************************************************/
  
 #include "cpl_serv.h"
 #include "geo_tiffp.h"
@@ -266,6 +80,46 @@ int GTIFGetPCSInfo( int nPCSCode, char **ppszEPSGName,
     char	**papszRecord;
     char	szSearchKey[24];
     const char	*pszFilename;
+    int         nDatum;
+    int         nZone;
+
+    int Proj = GTIFPCSToMapSys( nPCSCode, &nDatum, &nZone );
+    if ((Proj == MapSys_UTM_North || Proj == MapSys_UTM_South) &&
+        nDatum != KvUserDefined)
+    {
+        const char* pszDatumName = NULL;
+        switch (nDatum)
+        {
+            case GCS_NAD27: pszDatumName = "NAD27"; break;
+            case GCS_NAD83: pszDatumName = "NAD83"; break;
+            case GCS_WGS_72: pszDatumName = "WGS 72"; break;
+            case GCS_WGS_72BE: pszDatumName = "WGS 72BE"; break;
+            case GCS_WGS_84: pszDatumName = "WGS 84"; break;
+            default: break;
+        }
+
+        if (pszDatumName)
+        {
+            if (ppszEPSGName)
+            {
+                char szEPSGName[64];
+                sprintf(szEPSGName, "%s / UTM zone %d%c",
+                        pszDatumName, nZone, (Proj == MapSys_UTM_North) ? 'N' : 'S');
+                *ppszEPSGName = CPLStrdup(szEPSGName);
+            }
+
+            if (pnProjOp)
+                *pnProjOp = ((Proj == MapSys_UTM_North) ? Proj_UTM_zone_1N - 1 : Proj_UTM_zone_1S - 1) + nZone;
+
+            if (pnUOMLengthCode)
+                *pnUOMLengthCode = 9001; /* Linear_Meter */
+
+            if (pnGeogCS)
+                *pnGeogCS = nDatum;
+
+            return TRUE;
+        }
+    }
 
 /* -------------------------------------------------------------------- */
 /*      Search the pcs.override table for this PCS.                     */
@@ -368,7 +222,7 @@ double GTIFAngleToDD( double dfAngle, int nUOMAngle )
         sprintf( szAngleString, "%12.7f", dfAngle );
         dfAngle = GTIFAngleStringToDD( szAngleString, nUOMAngle );
     }
-    else
+    else if ( nUOMAngle != KvUserDefined )
     {
         double		dfInDegrees = 1.0;
         
@@ -474,6 +328,51 @@ int GTIFGetGCSInfo( int nGCSCode, char ** ppszName,
     int		nDatum, nPM, nUOMAngle;
     const char *pszFilename;
 
+/* -------------------------------------------------------------------- */
+/*      Handle some "well known" GCS codes directly                     */
+/* -------------------------------------------------------------------- */
+    const char * pszName = NULL;
+    nPM = PM_Greenwich;
+    nUOMAngle = Angular_DMS_Hemisphere; 
+    if( nGCSCode == GCS_NAD27 )
+    {
+        nDatum = Datum_North_American_Datum_1927;
+        pszName = "NAD27";
+    }
+    else if( nGCSCode == GCS_NAD83 )
+    {
+        nDatum = Datum_North_American_Datum_1983;
+        pszName = "NAD83";
+    }
+    else if( nGCSCode == GCS_WGS_84 )
+    {
+        nDatum = Datum_WGS84;
+        pszName = "WGS 84";
+    }
+    else if( nGCSCode == GCS_WGS_72 )
+    {
+        nDatum = Datum_WGS72;
+        pszName = "WGS 72";
+    }
+    else if ( nGCSCode == KvUserDefined )
+    {
+        return FALSE;
+    }
+
+    if (pszName != NULL)
+    {
+        if( ppszName != NULL )
+            *ppszName = CPLStrdup( pszName );
+        if( pnDatum != NULL )
+            *pnDatum = (short) nDatum;
+        if( pnPM != NULL )
+            *pnPM = (short) nPM;
+        if( pnUOMAngle != NULL )
+            *pnUOMAngle = (short) nUOMAngle;
+
+        return TRUE;
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Search the database for the corresponding datum code.           */
 /* -------------------------------------------------------------------- */
@@ -492,48 +391,9 @@ int GTIFGetGCSInfo( int nGCSCode, char ** ppszName,
                                    CC_Integer, "DATUM_CODE" ) );
     }
 
-/* -------------------------------------------------------------------- */
-/*      Handle some "well known" GCS codes directly if the table        */
-/*      wasn't found.                                                   */
-/* -------------------------------------------------------------------- */
     if( nDatum < 1 )
     {
-        const char * pszName = NULL;
-        nPM = PM_Greenwich;
-        nUOMAngle = Angular_DMS_Hemisphere; 
-        if( nGCSCode == GCS_NAD27 )
-        {
-            nDatum = Datum_North_American_Datum_1927;
-            pszName = "NAD27";
-        }
-        else if( nGCSCode == GCS_NAD83 )
-        {
-            nDatum = Datum_North_American_Datum_1983;
-            pszName = "NAD83";
-        }
-        else if( nGCSCode == GCS_WGS_84 )
-        {
-            nDatum = Datum_WGS84;
-            pszName = "WGS 84";
-        }
-        else if( nGCSCode == GCS_WGS_72 )
-        {
-            nDatum = Datum_WGS72;
-            pszName = "WGS 72";
-        }
-        else
-            return FALSE;
-
-        if( ppszName != NULL )
-            *ppszName = CPLStrdup( pszName );
-        if( pnDatum != NULL )
-            *pnDatum = (short) nDatum;
-        if( pnPM != NULL )
-            *pnPM = (short) nPM;
-        if( pnUOMAngle != NULL )
-            *pnUOMAngle = (short) nUOMAngle;
-
-        return TRUE;
+        return FALSE;
     }
 
     if( pnDatum != NULL )
@@ -594,56 +454,45 @@ int GTIFGetEllipsoidInfo( int nEllipseCode, char ** ppszName,
     char	szSearchKey[24];
     double	dfSemiMajor, dfToMeters = 1.0;
     int		nUOMLength;
-    
-/* -------------------------------------------------------------------- */
-/*      Get the semi major axis.                                        */
-/* -------------------------------------------------------------------- */
-    sprintf( szSearchKey, "%d", nEllipseCode );
-
-    dfSemiMajor =
-        atof(CSVGetField( CSVFilename("ellipsoid.csv" ),
-                          "ELLIPSOID_CODE", szSearchKey, CC_Integer,
-                          "SEMI_MAJOR_AXIS" ) );
+    const char* pszFilename;
 
 /* -------------------------------------------------------------------- */
 /*      Try some well known ellipsoids.                                 */
 /* -------------------------------------------------------------------- */
-    if( dfSemiMajor == 0.0 )
+    double     dfInvFlattening, dfSemiMinor;
+    const char *pszName = NULL;
+    
+    if( nEllipseCode == Ellipse_Clarke_1866 )
     {
-        double     dfInvFlattening, dfSemiMinor;
-        const char *pszName = NULL;
-        
-        if( nEllipseCode == Ellipse_Clarke_1866 )
-        {
-            pszName = "Clarke 1866";
-            dfSemiMajor = 6378206.4;
-            dfSemiMinor = 6356583.8;
-            dfInvFlattening = 0.0;
-        }
-        else if( nEllipseCode == Ellipse_GRS_1980 )
-        {
-            pszName = "GRS 1980";
-            dfSemiMajor = 6378137.0;
-            dfSemiMinor = 0.0;
-            dfInvFlattening = 298.257222101;
-        }
-        else if( nEllipseCode == Ellipse_WGS_84 )
-        {
-            pszName = "WGS 84";
-            dfSemiMajor = 6378137.0;
-            dfSemiMinor = 0.0;
-            dfInvFlattening = 298.257223563;
-        }
-        else if( nEllipseCode == 7043 )
-        {
-            pszName = "WGS 72";
-            dfSemiMajor = 6378135.0;
-            dfSemiMinor = 0.0;
-            dfInvFlattening = 298.26;
-        }
-        else
-            return FALSE;
+        pszName = "Clarke 1866";
+        dfSemiMajor = 6378206.4;
+        dfSemiMinor = 6356583.8;
+        dfInvFlattening = 0.0;
+    }
+    else if( nEllipseCode == Ellipse_GRS_1980 )
+    {
+        pszName = "GRS 1980";
+        dfSemiMajor = 6378137.0;
+        dfSemiMinor = 0.0;
+        dfInvFlattening = 298.257222101;
+    }
+    else if( nEllipseCode == Ellipse_WGS_84 )
+    {
+        pszName = "WGS 84";
+        dfSemiMajor = 6378137.0;
+        dfSemiMinor = 0.0;
+        dfInvFlattening = 298.257223563;
+    }
+    else if( nEllipseCode == 7043 )
+    {
+        pszName = "WGS 72";
+        dfSemiMajor = 6378135.0;
+        dfSemiMinor = 0.0;
+        dfInvFlattening = 298.26;
+    }
 
+    if (pszName != NULL)
+    {
         if( dfSemiMinor == 0.0 )
             dfSemiMinor = dfSemiMajor * (1 - 1.0/dfInvFlattening);
 
@@ -657,10 +506,26 @@ int GTIFGetEllipsoidInfo( int nEllipseCode, char ** ppszName,
         return TRUE;
     }
 
+/* -------------------------------------------------------------------- */
+/*      Get the semi major axis.                                        */
+/* -------------------------------------------------------------------- */
+    sprintf( szSearchKey, "%d", nEllipseCode );
+    pszFilename = CSVFilename("ellipsoid.csv" );
+
+    dfSemiMajor =
+        atof(CSVGetField( pszFilename,
+                          "ELLIPSOID_CODE", szSearchKey, CC_Integer,
+                          "SEMI_MAJOR_AXIS" ) );
+
+    if( dfSemiMajor == 0.0 )
+    {
+        return FALSE;
+    }
+
 /* -------------------------------------------------------------------- */
 /*	Get the translation factor into meters.				*/
 /* -------------------------------------------------------------------- */
-    nUOMLength = atoi(CSVGetField( CSVFilename("ellipsoid.csv" ),
+    nUOMLength = atoi(CSVGetField( pszFilename,
                                    "ELLIPSOID_CODE", szSearchKey, CC_Integer,
                                    "UOM_CODE" ));
     GTIFGetUOMLengthInfo( nUOMLength, NULL, &dfToMeters );
@@ -677,7 +542,7 @@ int GTIFGetEllipsoidInfo( int nEllipseCode, char ** ppszName,
     if( pdfSemiMinor != NULL )
     {
         *pdfSemiMinor =
-            atof(CSVGetField( CSVFilename("ellipsoid.csv" ),
+            atof(CSVGetField( pszFilename,
                               "ELLIPSOID_CODE", szSearchKey, CC_Integer,
                               "SEMI_MINOR_AXIS" )) * dfToMeters;
 
@@ -686,7 +551,7 @@ int GTIFGetEllipsoidInfo( int nEllipseCode, char ** ppszName,
             double	dfInvFlattening;
             
             dfInvFlattening = 
-                atof(CSVGetField( CSVFilename("ellipsoid.csv" ),
+                atof(CSVGetField( pszFilename,
                                   "ELLIPSOID_CODE", szSearchKey, CC_Integer,
                                   "INV_FLATTENING" ));
             *pdfSemiMinor = dfSemiMajor * (1 - 1.0/dfInvFlattening);
@@ -698,7 +563,7 @@ int GTIFGetEllipsoidInfo( int nEllipseCode, char ** ppszName,
 /* -------------------------------------------------------------------- */
     if( ppszName != NULL )
         *ppszName =
-            CPLStrdup(CSVGetField( CSVFilename("ellipsoid.csv" ),
+            CPLStrdup(CSVGetField( pszFilename,
                                    "ELLIPSOID_CODE", szSearchKey, CC_Integer,
                                    "ELLIPSOID_NAME" ));
     
@@ -717,7 +582,7 @@ int GTIFGetPMInfo( int nPMCode, char ** ppszName, double *pdfOffset )
 {
     char	szSearchKey[24];
     int		nUOMAngle;
-    const char *pszFilename = CSVFilename("prime_meridian.csv");
+    const char *pszFilename;
 
 /* -------------------------------------------------------------------- */
 /*      Use a special short cut for Greenwich, since it is so common.   */
@@ -734,6 +599,7 @@ int GTIFGetPMInfo( int nPMCode, char ** ppszName, double *pdfOffset )
 /* -------------------------------------------------------------------- */
 /*      Search the database for the corresponding datum code.           */
 /* -------------------------------------------------------------------- */
+    pszFilename = CSVFilename("prime_meridian.csv");
     sprintf( szSearchKey, "%d", nPMCode );
 
     nUOMAngle =
@@ -780,13 +646,50 @@ int GTIFGetDatumInfo( int nDatumCode, char ** ppszName, short * pnEllipsoid )
 {
     char	szSearchKey[24];
     int		nEllipsoid;
-    const char *pszFilename = CSVFilename( "datum.csv" );
+    const char *pszFilename;
     FILE       *fp;
+    const char *pszName = NULL;
+
+/* -------------------------------------------------------------------- */
+/*      Handle a few built-in datums.                                   */
+/* -------------------------------------------------------------------- */
+    if( nDatumCode == Datum_North_American_Datum_1927 )
+    {
+        nEllipsoid = Ellipse_Clarke_1866;
+        pszName = "North American Datum 1927";
+    }
+    else if( nDatumCode == Datum_North_American_Datum_1983 )
+    {
+        nEllipsoid = Ellipse_GRS_1980;
+        pszName = "North American Datum 1983";
+    }
+    else if( nDatumCode == Datum_WGS84 )
+    {
+        nEllipsoid = Ellipse_WGS_84;
+        pszName = "World Geodetic System 1984";
+    }
+    else if( nDatumCode == Datum_WGS72 )
+    {
+        nEllipsoid = 7043; /* WGS72 */
+        pszName = "World Geodetic System 1972";
+    }
+
+    if (pszName != NULL)
+    {
+        if( pnEllipsoid != NULL )
+            *pnEllipsoid = (short) nEllipsoid;
+
+        if( ppszName != NULL )
+            *ppszName = CPLStrdup( pszName );
+
+        return TRUE;
+    }
 
 /* -------------------------------------------------------------------- */
 /*      If we can't find datum.csv then gdal_datum.csv is an            */
 /*      acceptable fallback.  Mostly this is for GDAL.                  */
 /* -------------------------------------------------------------------- */
+    pszFilename = CSVFilename( "datum.csv" );
     if( (fp = VSIFOpen(pszFilename,"r")) == NULL )
     {
         if( (fp = VSIFOpen(CSVFilename("gdal_datum.csv"), "r")) != NULL )
@@ -809,44 +712,10 @@ int GTIFGetDatumInfo( int nDatumCode, char ** ppszName, short * pnEllipsoid )
 
     if( pnEllipsoid != NULL )
         *pnEllipsoid = (short) nEllipsoid;
-    
-/* -------------------------------------------------------------------- */
-/*      Handle a few built-in datums.                                   */
-/* -------------------------------------------------------------------- */
+
     if( nEllipsoid < 1 )
     {
-        const char *pszName = NULL;
-        
-        if( nDatumCode == Datum_North_American_Datum_1927 )
-        {
-            nEllipsoid = Ellipse_Clarke_1866;
-            pszName = "North American Datum 1927";
-        }
-        else if( nDatumCode == Datum_North_American_Datum_1983 )
-        {
-            nEllipsoid = Ellipse_GRS_1980;
-            pszName = "North American Datum 1983";
-        }
-        else if( nDatumCode == Datum_WGS84 )
-        {
-            nEllipsoid = Ellipse_WGS_84;
-            pszName = "World Geodetic System 1984";
-        }
-        else if( nDatumCode == Datum_WGS72 )
-        {
-            nEllipsoid = 7043; /* WGS7 */
-            pszName = "World Geodetic System 1972";
-        }
-        else
-            return FALSE;
-
-        if( pnEllipsoid != NULL )
-            *pnEllipsoid = (short) nEllipsoid;
-
-        if( ppszName != NULL )
-            *ppszName = CPLStrdup( pszName );
-
-        return TRUE;
+        return FALSE;
     }
 
 /* -------------------------------------------------------------------- */
@@ -969,9 +838,71 @@ int GTIFGetUOMAngleInfo( int nUOMAngleCode,
 {
     const char	*pszUOMName = NULL;
     double	dfInDegrees = 1.0;
-    const char *pszFilename = CSVFilename( "unit_of_measure.csv" );
+    const char *pszFilename;
     char	szSearchKey[24];
 
+    switch( nUOMAngleCode )
+    {
+      case 9101:
+        pszUOMName = "radian";
+        dfInDegrees = 180.0 / PI;
+        break;
+
+      case 9102:
+      case 9107:
+      case 9108:
+      case 9110:
+      case 9122:
+        pszUOMName = "degree";
+        dfInDegrees = 1.0;
+        break;
+
+      case 9103:
+        pszUOMName = "arc-minute";
+        dfInDegrees = 1 / 60.0;
+        break;
+
+      case 9104:
+        pszUOMName = "arc-second";
+        dfInDegrees = 1 / 3600.0;
+        break;
+
+      case 9105:
+        pszUOMName = "grad";
+        dfInDegrees = 180.0 / 200.0;
+        break;
+
+      case 9106:
+        pszUOMName = "gon";
+        dfInDegrees = 180.0 / 200.0;
+        break;
+
+      case 9109:
+        pszUOMName = "microradian";
+        dfInDegrees = 180.0 / (PI * 1000000.0);
+        break;
+
+      default:
+        break;
+    }
+    
+    if (pszUOMName)
+    {
+        if( ppszUOMName != NULL )
+        {
+            if( pszUOMName != NULL )
+                *ppszUOMName = CPLStrdup( pszUOMName );
+            else
+                *ppszUOMName = NULL;
+        }
+
+        if( pdfInDegrees != NULL )
+            *pdfInDegrees = dfInDegrees;
+
+        return TRUE;
+    }
+
+    pszFilename = CSVFilename( "unit_of_measure.csv" );
     sprintf( szSearchKey, "%d", nUOMAngleCode );
     pszUOMName = CSVGetField( pszFilename,
                               "UOM_CODE", szSearchKey, CC_Integer,
@@ -1002,66 +933,10 @@ int GTIFGetUOMAngleInfo( int nUOMAngleCode,
             dfInRadians = (dfFactorB / dfFactorC);
             dfInDegrees = dfInRadians * 180.0 / PI;
         }
-                          
-
-        /* We do a special override of some of the DMS formats name */
-        if( nUOMAngleCode == 9102 || nUOMAngleCode == 9107
-            || nUOMAngleCode == 9108 || nUOMAngleCode == 9110
-            || nUOMAngleCode == 9122 )
-        {
-            dfInDegrees = 1.0;
-            pszUOMName = "degree";
-        }
     }
-
-/* -------------------------------------------------------------------- */
-/*      Otherwise handle a few well known units directly.               */
-/* -------------------------------------------------------------------- */
     else
     {
-        switch( nUOMAngleCode )
-        {
-          case 9101:
-            pszUOMName = "radian";
-            dfInDegrees = 180.0 / PI;
-            break;
-        
-          case 9102:
-          case 9107:
-          case 9108:
-          case 9110:
-            pszUOMName = "degree";
-            dfInDegrees = 1.0;
-            break;
-
-          case 9103:
-            pszUOMName = "arc-minute";
-            dfInDegrees = 1 / 60.0;
-            break;
-
-          case 9104:
-            pszUOMName = "arc-second";
-            dfInDegrees = 1 / 3600.0;
-            break;
-        
-          case 9105:
-            pszUOMName = "grad";
-            dfInDegrees = 180.0 / 200.0;
-            break;
-
-          case 9106:
-            pszUOMName = "gon";
-            dfInDegrees = 180.0 / 200.0;
-            break;
-        
-          case 9109:
-            pszUOMName = "microradian";
-            dfInDegrees = 180.0 / (PI * 1000000.0);
-            break;
-
-          default:
-            return FALSE;
-        }
+        return FALSE;
     }
 
 /* -------------------------------------------------------------------- */
@@ -1109,6 +984,14 @@ static int EPSGProjMethodToCTProjMethod( int nEPSG )
 
       case 9805:
         return( CT_Mercator );  /* 1SP and 2SP not differentiated */
+        
+      /* Mercator 1SP (Spherical) For EPSG:3785 */
+      case 9841:
+        return( CT_Mercator );  /* 1SP and 2SP not differentiated */
+        
+      /* Google Mercator For EPSG:3857 */
+      case 1024:
+        return( CT_Mercator );  /* 1SP and 2SP not differentiated */
 
       case 9806:
         return( CT_CassiniSoldner );
@@ -1143,6 +1026,9 @@ static int EPSGProjMethodToCTProjMethod( int nEPSG )
       case 9816: /* tunesia mining grid has no counterpart */
         return( KvUserDefined );
 
+      case 9820:
+        return( CT_LambertAzimEqualArea );
+
       case 9822:
         return( CT_AlbersEqualArea );
     }
@@ -1284,6 +1170,18 @@ static int SetGTParmIds( int nCTProjection,
         /* EPSG codes? */
         return TRUE;
 
+      case CT_LambertAzimEqualArea:
+        panProjParmId[0] = ProjCenterLatGeoKey;
+        panProjParmId[1] = ProjCenterLongGeoKey;
+        panProjParmId[5] = ProjFalseEastingGeoKey;
+        panProjParmId[6] = ProjFalseNorthingGeoKey;
+
+        panEPSGCodes[0] = EPSGNatOriginLat;
+        panEPSGCodes[1] = EPSGNatOriginLong;
+        panEPSGCodes[5] = EPSGFalseEasting;
+        panEPSGCodes[6] = EPSGFalseNorthing;
+        return TRUE;
+
       default:
         return( FALSE );
     }
@@ -1309,12 +1207,54 @@ int GTIFGetProjTRFInfo( /* COORD_OP_CODE from coordinate_operation.csv */
     double	adfProjParms[7];
     char	szTRFCode[16];
     int         nCTProjMethod;
-    char       *pszFilename = CPLStrdup(CSVFilename("projop_wparm.csv"));
+    char       *pszFilename;
+    
+    if ((nProjTRFCode >= Proj_UTM_zone_1N && nProjTRFCode <= Proj_UTM_zone_60N) ||
+        (nProjTRFCode >= Proj_UTM_zone_1S && nProjTRFCode <= Proj_UTM_zone_60S))
+    {
+        int bNorth;
+        int nZone;
+        if (nProjTRFCode <= Proj_UTM_zone_60N)
+        {
+            bNorth = TRUE;
+            nZone = nProjTRFCode - Proj_UTM_zone_1N + 1;
+        }
+        else
+        {
+            bNorth = FALSE;
+            nZone = nProjTRFCode - Proj_UTM_zone_1S + 1;
+        }
+
+        if (ppszProjTRFName)
+        {
+            char szProjTRFName[64];
+            sprintf(szProjTRFName, "UTM zone %d%c",
+                    nZone, (bNorth) ? 'N' : 'S');
+            *ppszProjTRFName = CPLStrdup(szProjTRFName);
+        }
+
+        if (pnProjMethod)
+            *pnProjMethod = 9807;
+
+        if (padfProjParms)
+        {
+            padfProjParms[0] = 0;
+            padfProjParms[1] = -183 + 6 * nZone;
+            padfProjParms[2] = 0;
+            padfProjParms[3] = 0;
+            padfProjParms[4] = 0.9996;
+            padfProjParms[5] = 500000;
+            padfProjParms[6] = (bNorth) ? 0 : 10000000;
+        }
+
+        return TRUE;
+    }
 
 /* -------------------------------------------------------------------- */
 /*      Get the proj method.  If this fails to return a meaningful      */
 /*      number, then the whole function fails.                          */
 /* -------------------------------------------------------------------- */
+    pszFilename = CPLStrdup(CSVFilename("projop_wparm.csv"));
     sprintf( szTRFCode, "%d", nProjTRFCode );
     nProjMethod =
         atoi( CSVGetField( pszFilename,
@@ -2040,6 +1980,7 @@ int GTIFGetDefn( GTIF * psGTIF, GTIFDefn * psDefn )
 /* -------------------------------------------------------------------- */
 /*      Initially we default all the information we can.                */
 /* -------------------------------------------------------------------- */
+    psDefn->DefnSet = 1;
     psDefn->Model = KvUserDefined;
     psDefn->PCS = KvUserDefined;
     psDefn->GCS = KvUserDefined;
@@ -2068,6 +2009,21 @@ int GTIFGetDefn( GTIF * psGTIF, GTIFDefn * psDefn )
     psDefn->MapSys = KvUserDefined;
     psDefn->Zone = 0;
 
+/* -------------------------------------------------------------------- */
+/*      Do we have any geokeys?                                         */
+/* -------------------------------------------------------------------- */
+    {
+        int     nKeyCount = 0;
+        int     anVersion[3];
+        GTIFDirectoryInfo( psGTIF, anVersion, &nKeyCount );
+
+        if( nKeyCount == 0 )
+        {
+            psDefn->DefnSet = 0;
+            return FALSE;
+        }
+    }
+
 /* -------------------------------------------------------------------- */
 /*	Try to get the overall model type.				*/
 /* -------------------------------------------------------------------- */
@@ -2341,6 +2297,15 @@ const char *GTIFDecToDMS( double dfAngle, const char * pszAxis,
 void GTIFPrintDefn( GTIFDefn * psDefn, FILE * fp )
 
 {
+/* -------------------------------------------------------------------- */
+/*      Do we have anything to report?                                  */
+/* -------------------------------------------------------------------- */
+    if( !psDefn->DefnSet )
+    {
+        fprintf( fp, "No GeoKeys found.\n" );
+        return;
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Get the PCS name if possible.                                   */
 /* -------------------------------------------------------------------- */
@@ -2500,8 +2465,6 @@ void GTIFPrintDefn( GTIFDefn * psDefn, FILE * fp )
                  psDefn->UOMLength, pszName, psDefn->UOMLengthInMeters );
         CPLFree( pszName );
     }
-
-    CSVDeaccess( NULL );
 }
 
 /************************************************************************/
