@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrpoint.cpp 16853 2009-04-26 12:22:10Z rouault $
+ * $Id: ogrpoint.cpp 20260 2010-08-11 20:39:54Z rouault $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  The Point geometry class.
@@ -30,7 +30,7 @@
 #include "ogr_geometry.h"
 #include "ogr_p.h"
 
-CPL_CVSID("$Id: ogrpoint.cpp 16853 2009-04-26 12:22:10Z rouault $");
+CPL_CVSID("$Id: ogrpoint.cpp 20260 2010-08-11 20:39:54Z rouault $");
 
 /************************************************************************/
 /*                              OGRPoint()                              */
@@ -244,6 +244,9 @@ OGRErr OGRPoint::importFromWkb( unsigned char * pabyData,
 
     if( bIs3D )
     {
+        if ( nSize < 29 && nSize != -1 )
+            return OGRERR_NOT_ENOUGH_DATA;
+
         memcpy( &z, pabyData + 5 + 16, 8 );
         if( OGR_SWAP( eByteOrder ) )
         {
@@ -337,30 +340,66 @@ OGRErr OGRPoint::importFromWkt( char ** ppszInput )
 /*      Check for EMPTY ... but treat like a point at 0,0.              */
 /* -------------------------------------------------------------------- */
     const char *pszPreScan;
+    int bHasZ = FALSE, bHasM = FALSE;
 
     pszPreScan = OGRWktReadToken( pszInput, szToken );
     if( EQUAL(szToken,"EMPTY") )
     {
-        *ppszInput = (char *) pszInput;
+        *ppszInput = (char *) pszPreScan;
         empty();
         return OGRERR_NONE;
     }
 
+/* -------------------------------------------------------------------- */
+/*      Check for Z, M or ZM. Will ignore the Measure                   */
+/* -------------------------------------------------------------------- */
+    else if( EQUAL(szToken,"Z") )
+    {
+        bHasZ = TRUE;
+    }
+    else if( EQUAL(szToken,"M") )
+    {
+        bHasM = TRUE;
+    }
+    else if( EQUAL(szToken,"ZM") )
+    {
+        bHasZ = TRUE;
+        bHasM = TRUE;
+    }
+
+    if (bHasZ || bHasM)
+    {
+        pszInput = pszPreScan;
+        pszPreScan = OGRWktReadToken( pszInput, szToken );
+        if( EQUAL(szToken,"EMPTY") )
+        {
+            *ppszInput = (char *) pszPreScan;
+            empty();
+            /* FIXME?: In theory we should store the dimension and M presence */
+            /* if we want to allow round-trip with ExportToWKT v1.2 */
+            return OGRERR_NONE;
+        }
+    }
+
     if( !EQUAL(szToken,"(") )
         return OGRERR_CORRUPT_DATA;
 
-    pszPreScan = OGRWktReadToken( pszPreScan, szToken );
-    if( EQUAL(szToken,"EMPTY") )
+    if ( !bHasZ && !bHasM )
     {
-        pszInput = OGRWktReadToken( pszPreScan, szToken );
-        
-        if( !EQUAL(szToken,")") )
-            return OGRERR_CORRUPT_DATA;
-        else
+        /* Test for old-style POINT(EMPTY) */
+        pszPreScan = OGRWktReadToken( pszPreScan, szToken );
+        if( EQUAL(szToken,"EMPTY") )
         {
-            *ppszInput = (char *) pszInput;
-            empty();
-            return OGRERR_NONE;
+            pszInput = OGRWktReadToken( pszPreScan, szToken );
+
+            if( !EQUAL(szToken,")") )
+                return OGRERR_CORRUPT_DATA;
+            else
+            {
+                *ppszInput = (char *) pszInput;
+                empty();
+                return OGRERR_NONE;
+            }
         }
     }
 
@@ -374,7 +413,11 @@ OGRErr OGRPoint::importFromWkt( char ** ppszInput )
     pszInput = OGRWktReadPoints( pszInput, &poPoints, &padfZ,
                                  &nMaxPoint, &nPoints );
     if( pszInput == NULL || nPoints != 1 )
+    {
+        CPLFree( poPoints );
+        CPLFree( padfZ );
         return OGRERR_CORRUPT_DATA;
+    }
 
     x = poPoints[0].x;
     y = poPoints[0].y;
@@ -383,10 +426,23 @@ OGRErr OGRPoint::importFromWkt( char ** ppszInput )
 
     if( padfZ != NULL )
     {
-        z = padfZ[0];						
-        nCoordDimension = 3;
+        /* If there's a 3rd value, and it is not a POINT M, */
+        /* then assume it is the Z */
+        if ((!(bHasM && !bHasZ)))
+        {
+            z = padfZ[0];
+            nCoordDimension = 3;
+        }
+        else
+            nCoordDimension = 2;
         CPLFree( padfZ );
     }
+    else if ( bHasZ )
+    {
+        /* In theory we should have a z coordinate for POINT Z */
+        /* oh well, let be tolerant */
+        nCoordDimension = 3;
+    }
     else
         nCoordDimension = 2;
 
@@ -542,3 +598,14 @@ OGRBoolean OGRPoint::IsEmpty(  ) const
 {
     return nCoordDimension == 0;
 }
+
+/************************************************************************/
+/*                               swapXY()                               */
+/************************************************************************/
+
+void OGRPoint::swapXY()
+{
+    double dfTemp = x;
+    x = y;
+    y = dfTemp;
+}
