@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrsfdriverregistrar.cpp 15208 2008-08-25 16:46:14Z rouault $
+ * $Id: ogrsfdriverregistrar.cpp 21447 2011-01-09 16:02:28Z rouault $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  The OGRSFDriverRegistrar class implementation.
@@ -31,11 +31,16 @@
 #include "ogr_api.h"
 #include "ogr_p.h"
 #include "cpl_multiproc.h"
+#include "swq.h"
 
-CPL_CVSID("$Id: ogrsfdriverregistrar.cpp 15208 2008-08-25 16:46:14Z rouault $");
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
+CPL_CVSID("$Id: ogrsfdriverregistrar.cpp 21447 2011-01-09 16:02:28Z rouault $");
 
 static void *hDRMutex = NULL;
-static OGRSFDriverRegistrar *poRegistrar = NULL;
+static OGRSFDriverRegistrar * volatile poRegistrar = NULL;
 
 static const char *pszUpdatableINST_DATA = 
 "__INST_DATA_TARGET:                                                                                                                                      ";
@@ -44,7 +49,7 @@ static const char *pszUpdatableINST_DATA =
 /************************************************************************/
 
 /**
- * Constructor
+ * \brief Constructor
  *
  * Normally the driver registrar is constucted by the 
  * OGRSFDriverRegistrar::GetRegistrar() accessor which ensures singleton
@@ -115,7 +120,7 @@ OGRSFDriverRegistrar::~OGRSFDriverRegistrar()
 /************************************************************************/
 
 /**
- * Cleanup all OGR related resources. 
+ * \brief Cleanup all OGR related resources. 
  *
  * This function will destroy the OGRSFDriverRegistrar along with all registered
  * drivers, and then cleanup long lived OSR (OGRSpatialReference) and CPL
@@ -142,6 +147,7 @@ void OGRCleanupAll()
         if( poRegistrar != NULL )
             delete poRegistrar;
         OSRCleanup();
+        swq_op_registrar::DeInitialize();
     }
 
     CPLFinderClean();
@@ -156,7 +162,7 @@ void OGRCleanupAll()
 /************************************************************************/
 
 /**
- * Fetch registrar.
+ * \brief Fetch registrar.
  *
  * This static method should be used to fetch the singleton 
  * registrar.  It will create a registrar if there is not already
@@ -168,11 +174,15 @@ void OGRCleanupAll()
 OGRSFDriverRegistrar *OGRSFDriverRegistrar::GetRegistrar()
 
 {
-    CPLMutexHolderD( &hDRMutex );
-    
     if( poRegistrar == NULL )
-        poRegistrar = new OGRSFDriverRegistrar();
+    {
+        CPLMutexHolderD( &hDRMutex );
 
+        if( poRegistrar == NULL )
+            poRegistrar = new OGRSFDriverRegistrar();
+    }
+   
+    CPLAssert( NULL != poRegistrar );
     return poRegistrar;
 }
 
@@ -187,6 +197,11 @@ OGRDataSource *OGRSFDriverRegistrar::Open( const char * pszName,
 {
     OGRDataSource       *poDS;
 
+#ifdef HAVE_READLINK
+    char szPointerFilename[2048];
+    int  bHasRetried = FALSE;
+#endif
+
     if( ppoDriver != NULL )
         *ppoDriver = NULL;
 
@@ -196,6 +211,9 @@ OGRDataSource *OGRSFDriverRegistrar::Open( const char * pszName,
 
     CPLAcquireMutex( hDRMutex, 0.1 );
 
+#ifdef HAVE_READLINK
+retry:
+#endif
     for( int iDriver = 0; iDriver < poRegistrar->nDrivers; iDriver++ )
     {
         OGRSFDriver *poDriver = poRegistrar->papoDrivers[iDriver];
@@ -224,6 +242,23 @@ OGRDataSource *OGRSFDriverRegistrar::Open( const char * pszName,
         CPLAcquireMutex( hDRMutex, 0.1 );
     }
 
+#ifdef HAVE_READLINK
+    if (!bHasRetried)
+    {
+        /* If someone creates a file with "ln -sf /vsicurl/http://svn.osgeo.org/gdal/trunk/autotest/ogr/data/poly.shp my_remote_poly.shp" */
+        /* we will be able to open it by passing my_remote_poly.shp */
+        /* This helps a lot for OGR based readers that only provide file explorers to open datasources */
+        int nBytes = readlink(pszName, szPointerFilename, sizeof(szPointerFilename));
+        if (nBytes != -1)
+        {
+            szPointerFilename[MIN(nBytes, (int)sizeof(szPointerFilename)-1)] = 0;
+            pszName = szPointerFilename;
+            bHasRetried = TRUE;
+            goto retry;
+        }
+    }
+#endif
+
     CPLReleaseMutex( hDRMutex );
 
     CPLDebug( "OGR", "OGROpen(%s) failed.", pszName );
@@ -542,7 +577,25 @@ void OGRSFDriverRegistrar::RegisterDriver( OGRSFDriver * poDriver )
             delete poDriver;
             return;
         }
-    }                                                   
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Skip and destroy drivers in the black list.                     */
+/* -------------------------------------------------------------------- */
+    char** papszSkipDrivers =
+            CSLTokenizeStringComplex(CPLGetConfigOption("OGR_SKIP", ""), ",", FALSE, FALSE);
+    char** iter = papszSkipDrivers;
+    while(*iter)
+    {
+        if (strcmp(*iter, poDriver->GetName()) == 0)
+        {
+            CSLDestroy(papszSkipDrivers);
+            delete poDriver;
+            return;
+        }
+        iter ++;
+    }
+    CSLDestroy(papszSkipDrivers);
 
 /* -------------------------------------------------------------------- */
 /*      Add to the end of the driver list.                              */
@@ -566,6 +619,46 @@ void OGRRegisterDriver( OGRSFDriverH hDriver )
         (OGRSFDriver *) hDriver );
 }
 
+/************************************************************************/
+/*                          DeregisterDriver()                          */
+/************************************************************************/
+
+void OGRSFDriverRegistrar::DeregisterDriver( OGRSFDriver * poDriver )
+
+{
+    CPLMutexHolderD( &hDRMutex );
+    int         i;
+
+    for( i = 0; i < nDrivers; i++ )
+    {
+        if( poDriver == papoDrivers[i] )
+            break;
+    }
+
+    if (i == nDrivers)
+        return;
+
+    while( i < nDrivers-1 )
+    {
+        papoDrivers[i] = papoDrivers[i+1];
+        i++;
+    }
+    nDrivers--;
+}
+
+/************************************************************************/
+/*                        OGRDeregisterDriver()                         */
+/************************************************************************/
+
+void OGRDeregisterDriver( OGRSFDriverH hDriver )
+
+{
+    VALIDATE_POINTER0( hDriver, "OGRDeregisterDriver" );
+
+    OGRSFDriverRegistrar::GetRegistrar()->DeregisterDriver( 
+        (OGRSFDriver *) hDriver );
+}
+
 /************************************************************************/
 /*                           GetDriverCount()                           */
 /************************************************************************/
@@ -653,7 +746,7 @@ OGRSFDriverH OGRGetDriverByName( const char *pszName )
 /************************************************************************/
 
 /**
- * Auto-load GDAL drivers from shared libraries.
+ * \brief Auto-load GDAL drivers from shared libraries.
  *
  * This function will automatically load drivers from shared libraries.  It
  * searches the "driver path" for .so (or .dll) files that start with the
@@ -669,12 +762,12 @@ OGRSFDriverH OGRGetDriverByName( const char *pszName )
  * If that is not set the following defaults are used:
  *
  * <ul>
- * <li> Linux/Unix: <prefix>/lib/gdalplugins is searched or 
+ * <li> Linux/Unix: &lt;prefix&gt;/lib/gdalplugins is searched or
  * /usr/local/lib/gdalplugins if the install prefix is not known.
- * <li> MacOSX: <prefix>/PlugIns is searched, or /usr/local/lib/gdalplugins if
+ * <li> MacOSX: &lt;prefix&gt;/PlugIns is searched, or /usr/local/lib/gdalplugins if
  * the install prefix is not known.  Also, the framework directory
  * /Library/Application Support/GDAL/PlugIns is searched.
- * <li> Win32: <prefix>/lib/gdalplugins if the prefix is known (normally it 
+ * <li> Win32: &lt;prefix&gt;/lib/gdalplugins if the prefix is known (normally it
  * is not), otherwise the gdalplugins subdirectory of the directory containing
  * the currently running executable is used. 
  * </ul>
@@ -731,8 +824,12 @@ void OGRSFDriverRegistrar::AutoLoadDrivers()
 #endif 
 
 #ifdef MACOSX_FRAMEWORK
+#define num2str(x) str(x)
+#define str(x) #x 
         papszSearchPath = CSLAddString( papszSearchPath, 
-                                        "/Library/Application Support/GDAL/PlugIns" );
+                                        "/Library/Application Support/GDAL/"
+                                        num2str(GDAL_VERSION_MAJOR) "."
+                                        num2str(GDAL_VERSION_MINOR) "/PlugIns" );
 #endif
 
     }
