@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrmysqltablelayer.cpp 14430 2008-05-10 18:42:32Z warmerdam $
+ * $Id: ogrmysqltablelayer.cpp 20460 2010-08-27 20:06:26Z rouault $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRMySQLTableLayer class.
@@ -32,7 +32,7 @@
 #include "cpl_string.h"
 #include "ogr_mysql.h"
 
-CPL_CVSID("$Id: ogrmysqltablelayer.cpp 14430 2008-05-10 18:42:32Z warmerdam $");
+CPL_CVSID("$Id: ogrmysqltablelayer.cpp 20460 2010-08-27 20:06:26Z rouault $");
 
 /************************************************************************/
 /*                         OGRMySQLTableLayer()                         */
@@ -104,14 +104,14 @@ OGRFeatureDefn *OGRMySQLTableLayer::ReadTableDefinition( const char *pszTable )
 
 {
     MYSQL_RES    *hResult;
-    char         szCommand[1024];
+    CPLString     osCommand;
     
 /* -------------------------------------------------------------------- */
 /*      Fire off commands to get back the schema of the table.          */
 /* -------------------------------------------------------------------- */
-    sprintf( szCommand, "DESCRIBE `%s`", pszTable );
+    osCommand.Printf("DESCRIBE `%s`", pszTable );
     pszGeomColumnTable = CPLStrdup(pszTable);
-    if( mysql_query( poDS->GetConn(), szCommand ) )
+    if( mysql_query( poDS->GetConn(), osCommand ) )
     {
         poDS->ReportError( "DESCRIBE Failed" );
         return FALSE;
@@ -277,7 +277,8 @@ OGRFeatureDefn *OGRMySQLTableLayer::ReadTableDefinition( const char *pszTable )
         }
         else if( EQUAL(pszType, "geometry") ) 
         {
-            pszGeomColumn = CPLStrdup(papszRow[0]);
+            if (pszGeomColumn == NULL)
+                pszGeomColumn = CPLStrdup(papszRow[0]);
             continue;
         }
         // Is this an integer primary key field?
@@ -316,11 +317,12 @@ OGRFeatureDefn *OGRMySQLTableLayer::ReadTableDefinition( const char *pszTable )
         // set to unknown first
         poDefn->SetGeomType( wkbUnknown );
         
-        sprintf(szCommand, "SELECT type, coord_dimension FROM geometry_columns WHERE f_table_name='%s'",
-                pszTable );
+        osCommand = "SELECT type, coord_dimension FROM geometry_columns WHERE f_table_name='";
+        osCommand += pszTable;
+        osCommand += "'";
         
         hResult = NULL;
-        if( !mysql_query( poDS->GetConn(), szCommand ) )
+        if( !mysql_query( poDS->GetConn(), osCommand ) )
             hResult = mysql_store_result( poDS->GetConn() );
 
         papszRow = NULL;
@@ -332,23 +334,7 @@ OGRFeatureDefn *OGRMySQLTableLayer::ReadTableDefinition( const char *pszTable )
 
             pszType = papszRow[0];
 
-            OGRwkbGeometryType nGeomType = wkbUnknown;
-
-            // check only standard OGC geometry types
-            if ( EQUAL(pszType, "POINT") )
-                nGeomType = wkbPoint;
-            else if ( EQUAL(pszType,"LINESTRING"))
-                nGeomType = wkbLineString;
-            else if ( EQUAL(pszType,"POLYGON"))
-                nGeomType = wkbPolygon;
-            else if ( EQUAL(pszType,"MULTIPOINT"))
-                nGeomType = wkbMultiPoint;
-            else if ( EQUAL(pszType,"MULTILINESTRING"))
-                nGeomType = wkbMultiLineString;
-            else if ( EQUAL(pszType,"MULTIPOLYGON"))
-                nGeomType = wkbMultiPolygon;
-            else if ( EQUAL(pszType,"GEOMETRYCOLLECTION"))
-                nGeomType = wkbGeometryCollection;
+            OGRwkbGeometryType nGeomType = OGRFromOGCGeomType(pszType);
 
             if( papszRow[1] != NULL && atoi(papszRow[1]) == 3 )
                 nGeomType = (OGRwkbGeometryType) (nGeomType | wkb25DBit);
@@ -863,7 +849,7 @@ OGRErr OGRMySQLTableLayer::CreateField( OGRFieldDefn *poFieldIn, int bApproxOK )
 {
 
     MYSQL_RES           *hResult=NULL;
-    char        		szCommand[1024];
+    CPLString            osCommand;
     
     char                szFieldType[256];
     OGRFieldDefn        oField( poFieldIn );
@@ -946,13 +932,13 @@ OGRErr OGRMySQLTableLayer::CreateField( OGRFieldDefn *poFieldIn, int bApproxOK )
         return OGRERR_FAILURE;
     }
 
-    sprintf( szCommand,
+    osCommand.Printf(
              "ALTER TABLE `%s` ADD COLUMN `%s` %s",
              poFeatureDefn->GetName(), oField.GetNameRef(), szFieldType );
 
-    if( mysql_query(poDS->GetConn(), szCommand ) )
+    if( mysql_query(poDS->GetConn(), osCommand ) )
     {
-        poDS->ReportError( szCommand );
+        poDS->ReportError( osCommand );
         return OGRERR_FAILURE;
     }
 
@@ -988,9 +974,9 @@ OGRFeature *OGRMySQLTableLayer::GetFeature( long nFeatureId )
 /*      interest.                                                       */
 /* -------------------------------------------------------------------- */
     char        *pszFieldList = BuildFields();
-    char        *pszCommand = (char *) CPLMalloc(strlen(pszFieldList)+2000);
+    CPLString    osCommand;
 
-    sprintf( pszCommand, 
+    osCommand.Printf(
              "SELECT %s FROM `%s` WHERE `%s` = %ld", 
              pszFieldList, poFeatureDefn->GetName(), pszFIDColumn, 
              nFeatureId );
@@ -999,12 +985,11 @@ OGRFeature *OGRMySQLTableLayer::GetFeature( long nFeatureId )
 /* -------------------------------------------------------------------- */
 /*      Issue the command.                                              */
 /* -------------------------------------------------------------------- */
-    if( mysql_query( poDS->GetConn(), pszCommand ) )
+    if( mysql_query( poDS->GetConn(), osCommand ) )
     {
-        poDS->ReportError( pszCommand );
+        poDS->ReportError( osCommand );
         return NULL;
     }
-    CPLFree( pszCommand );
 
     hResultSet = mysql_store_result( poDS->GetConn() );
     if( hResultSet == NULL )
