@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrogdilayer.cpp 14048 2008-03-20 18:47:21Z rouault $
+ * $Id: ogrogdilayer.cpp 20207 2010-08-06 22:07:29Z rouault $
  *
  * Project:  OGDI Bridge
  * Purpose:  Implements OGROGDILayer class.
@@ -56,7 +56,7 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogrogdilayer.cpp 14048 2008-03-20 18:47:21Z rouault $");
+CPL_CVSID("$Id: ogrogdilayer.cpp 20207 2010-08-06 22:07:29Z rouault $");
 
 /************************************************************************/
 /*                           OGROGDILayer()                            */
@@ -119,31 +119,26 @@ void OGROGDILayer::SetSpatialFilter( OGRGeometry * poGeomIn )
     if( !InstallFilter( poGeomIn ) )
         return;
 
-    if( m_poFilterGeom != NULL )
-    {
-        OGREnvelope     oEnv;
-        ecs_Result     *psResult;
+    ResetReading();
 
-        m_poFilterGeom->getEnvelope(&oEnv);
-
-        m_sFilterBounds.north = oEnv.MaxY;
-        m_sFilterBounds.south = oEnv.MinY;
-        m_sFilterBounds.east  = oEnv.MinX;
-        m_sFilterBounds.west  = oEnv.MaxX;
-
-        psResult = cln_SelectRegion( m_nClientID, &m_sFilterBounds);
-        if( ECSERROR(psResult) )
-        {
-            CPLError( CE_Failure, CPLE_AppDefined,
-                      "%s", psResult->message );
-            return;
-        }
-    }
-
-    m_iNextShapeId = 0;
     m_nTotalShapeCount = -1;
 }
 
+/************************************************************************/
+/*                         SetAttributeFilter()                         */
+/************************************************************************/
+
+OGRErr OGROGDILayer::SetAttributeFilter( const char *pszQuery )
+{
+    OGRErr eErr = OGRLayer::SetAttributeFilter(pszQuery);
+
+    ResetReading();
+
+    m_nTotalShapeCount = -1;
+
+    return eErr;
+}
+
 /************************************************************************/
 /*                            ResetReading()                            */
 /************************************************************************/
@@ -166,7 +161,40 @@ void OGROGDILayer::ResetReading()
         return;
     }
 
+    /* Reset spatial filter */
+    if( m_poFilterGeom != NULL )
+    {
+        OGREnvelope     oEnv;
+
+        m_poFilterGeom->getEnvelope(&oEnv);
+
+        m_sFilterBounds.north = oEnv.MaxY;
+        m_sFilterBounds.south = oEnv.MinY;
+        m_sFilterBounds.east  = oEnv.MinX;
+        m_sFilterBounds.west  = oEnv.MaxX;
+
+        psResult = cln_SelectRegion( m_nClientID, &m_sFilterBounds);
+        if( ECSERROR(psResult) )
+        {
+            CPLError( CE_Failure, CPLE_AppDefined,
+                      "%s", psResult->message );
+            return;
+        }
+    }
+    else
+    {
+        /* Reset to global bounds */
+        psResult = cln_SelectRegion( m_nClientID, m_poODS->GetGlobalBounds() );
+        if( ECSERROR(psResult) )
+        {
+            CPLError( CE_Failure, CPLE_AppDefined,
+                      "%s", psResult->message );
+            return;
+        }
+    }
+
     m_iNextShapeId = 0;
+    m_nFilteredOutShapes = 0;
 }
 
 /************************************************************************/
@@ -180,6 +208,14 @@ OGRFeature *OGROGDILayer::GetNextFeature()
     ecs_Result  *psResult;
     int         i;
 
+    /* Reset reading if we are not the current layer */
+    /* WARNING : this does not allow interleaved reading of layers */ 
+    if( m_poODS->GetCurrentLayer() != this )
+    {
+        m_poODS->SetCurrentLayer(this);
+        ResetReading();
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Retrieve object from OGDI server and create new feature         */
 /* -------------------------------------------------------------------- */
@@ -188,7 +224,7 @@ OGRFeature *OGROGDILayer::GetNextFeature()
     if (! ECSSUCCESS(psResult))
     {
         // We probably reached EOF... keep track of shape count.
-        m_nTotalShapeCount = m_iNextShapeId;
+        m_nTotalShapeCount = m_iNextShapeId - m_nFilteredOutShapes;
         return NULL;
     }
    
@@ -205,6 +241,7 @@ OGRFeature *OGROGDILayer::GetNextFeature()
         ecs_Point       *psPoint = &(ECSGEOM(psResult).point);
         OGRPoint        *poOGRPoint = new OGRPoint(psPoint->c.x, psPoint->c.y);
 
+        poOGRPoint->assignSpatialReference(m_poSpatialRef);
         poFeature->SetGeometryDirectly(poOGRPoint);
     }
     else if (m_eFamily == Line)
@@ -219,6 +256,7 @@ OGRFeature *OGROGDILayer::GetNextFeature()
             poOGRLine->setPoint(i, psLine->c.c_val[i].x, psLine->c.c_val[i].y);
         }
 
+        poOGRLine->assignSpatialReference(m_poSpatialRef);
         poFeature->SetGeometryDirectly(poOGRLine);
     }
     else if (m_eFamily == Area)
@@ -244,6 +282,7 @@ OGRFeature *OGROGDILayer::GetNextFeature()
         // __TODO__
         // When OGR supports polygon centroids then we should carry them here
 
+        poOGRPolygon->assignSpatialReference(m_poSpatialRef);
         poFeature->SetGeometryDirectly(poOGRPolygon);
     }
     else if (m_eFamily == Text)
@@ -254,6 +293,7 @@ OGRFeature *OGROGDILayer::GetNextFeature()
         ecs_Text       *psText = &(ECSGEOM(psResult).text);
         OGRPoint        *poOGRPoint = new OGRPoint(psText->c.x, psText->c.y);
 
+        poOGRPoint->assignSpatialReference(m_poSpatialRef);
         poFeature->SetGeometryDirectly(poOGRPoint);
     }
     else
@@ -321,6 +361,7 @@ OGRFeature *OGROGDILayer::GetNextFeature()
         || (m_poFilterGeom != NULL 
             && !FilterGeometry( poFeature->GetGeometryRef() ) ) )
     {
+        m_nFilteredOutShapes ++;
         delete poFeature;
         goto TryAgain;
     }
@@ -393,11 +434,8 @@ int OGROGDILayer::TestCapability( const char * pszCap )
 /*      For now just return FALSE for everything.                       */
 /* -------------------------------------------------------------------- */
 #ifdef __TODO__
-    if( EQUAL(pszCap,OLCRandomRead) )
-        return TRUE;
-
-    else if( EQUAL(pszCap,OLCFastFeatureCount) )
-        return m_poFilterGeom == NULL;
+    if( EQUAL(pszCap,OLCFastFeatureCount) )
+        return m_poFilterGeom == NULL && m_poAttrQuery == NULL;
 
     else if( EQUAL(pszCap,OLCFastSpatialFilter) )
         return FALSE;
@@ -406,7 +444,11 @@ int OGROGDILayer::TestCapability( const char * pszCap )
         return FALSE;
 #endif
 
-    return FALSE;
+    if( EQUAL(pszCap,OLCRandomRead) )
+        return TRUE;
+
+    else
+        return FALSE;
 }
 
 
@@ -452,10 +494,27 @@ void OGROGDILayer::BuildFeatureDefn()
         eLayerGeomType = wkbUnknown;
         break;
     }
-
-    m_poFeatureDefn = new OGRFeatureDefn(CPLSPrintf("%s_%s", 
+    
+    char* pszFeatureDefnName;
+    if (m_poODS->LaunderLayerNames())
+    {
+        pszFeatureDefnName = CPLStrdup(m_pszOGDILayerName);
+        char* pszAt = strchr(pszFeatureDefnName, '@');
+        if (pszAt)
+            *pszAt = '_';
+        char* pszLeftParenthesis = strchr(pszFeatureDefnName, '(');
+        if (pszLeftParenthesis)
+            *pszLeftParenthesis = '\0';
+    }
+    else
+        pszFeatureDefnName = CPLStrdup(CPLSPrintf("%s_%s", 
                                                     m_pszOGDILayerName, 
                                                     pszGeomName ));
+
+    m_poFeatureDefn = new OGRFeatureDefn(pszFeatureDefnName);
+    CPLFree(pszFeatureDefnName);
+    pszFeatureDefnName = NULL;
+    
     m_poFeatureDefn->SetGeomType(eLayerGeomType);
     m_poFeatureDefn->Reference();
 
